

def korean_character_ratio(text: str, ignore_whitespace: bool = True) -> float:
    if ignore_whitespace:
        text = ''.join(text.split())

    korean_characters = len([
        c for c in text if ord('가') <= ord(c) <= ord('힣')])
    return korean_characters / len(text) if len(text) > 0 else 0


def is_normal_character(c: str) -> bool:
    return (ord('0') <= ord(c) <= ord('9')
            or ord('a') <= ord(c) <= ord('z')
            or ord('A') <= ord(c) <= ord('Z')
            or ord('가') <= ord(c) <= ord('힣'))



import ijson
import zipfile
from typing import Iterable, IO
from langumo.building import Parser
from langumo.utils import AuxiliaryFile


class ModuBaseParser(Parser):
    def _extract_json_file(self, fp: IO[str]) -> Iterable[str]:
        paragraph = []

        for prefix, event, value in ijson.parse(fp):
            if prefix == 'document.item.paragraph.item.form':
                paragraph.append(value)
            elif prefix == 'document.item.paragraph' and event == 'end_array':
                yield '\n'.join(paragraph)
                paragraph.clear()

    def extract(self, raw: AuxiliaryFile) -> Iterable[str]:
        with zipfile.ZipFile(raw.name, 'r') as zfp:
            for filename in zfp.namelist():
                if filename.startswith('#'):
                    continue

                with zfp.open(filename, 'r') as fp:
                    yield from self._extract_json_file(fp)



import re


class Renderer:
    def __init__(self, *rules: str):
        self.patterns = [re.compile(rule, re.IGNORECASE | re.MULTILINE)
                         for rule in rules]

    def replacement(self, match: re.Match) -> str:
        raise NotImplementedError()

    def render(self, source: str) -> str:
        for pattern in self.patterns:
            source = pattern.sub(self.replacement, source)
        return source


class InlineRenderer(Renderer):
    def replacement(self, match: re.Match) -> str:
        return match.groupdict().get('text', '')


class PaddedInlineRenderer(Renderer):
    def replacement(self, match: re.Match) -> str:
        return f" {match.groupdict().get('text', '')} "


class MacroRenderer(Renderer):
    def __init__(self):
        super().__init__(
            (r'\[\[(date|datetime|br|include|목차|tableofcontents|각주|footnote'
             r'|pagecount|youtube|nicovideo|wikicommons)(?:\(.+?\))?\]\]'),
            (r'\[(date|datetime|br|include|목차|tableofcontents|각주|footnote'
             r'|pagecount|youtube)(?:\(.+?\))?\]'),
            r'\[\[분류\:.*?\]\]')

    def replacement(self, match: re.Match) -> str:
        return ' '


class HTMLRenderer(Renderer):
    def __init__(self):
        super().__init__(
            r'\{\{\{\#\!html[ \t\n]+(?P<text>[\s\S]*?)\}\}\}',
            r'\[\[html[ \t]*\((?P<text>[^\)]*)\)\]\]')
        self.remove_tags = re.compile(r'</?[^>]+>')

    def replacement(self, match: re.Match) -> str:
        return self.remove_tags.sub('', match.group('text'))


class ImageRenderer(PaddedInlineRenderer):
    def __init__(self):
        super().__init__(
            (r'\[\[파일\:[ \t]*.*?\.(?:jpg|jpeg|png|gif)(?:\|[\&a-z\=\d\%]*)?'
             r'\]\]'),
            r'(?:https?|ftp).*?\.(?:jpg|jpeg|png|gif)(?:[\?\&a-z\=\d\%]*)?',
            (r'attachment\:[ \t]*[\"\']?.*?\.(?:jpg|jpeg|png|gif)'
             r'(?:[\?\&a-z\=\d\%]*)?[\"\']?'))


class HyperLinkRenderer(InlineRenderer):
    def __init__(self):
        super().__init__(
            r'\[\[[^\|\]]*\|(?P<text>[^\]]*)\]\]',
            r'\[\[\\(?P<text>\#[^\]]*?)(?:\#[^\]]*)?\]\]',
            r'\[\[(?P<text>[^\#][^\]]*?)(?:\#[^\]]*)?\]\]',
            r'\[\[\#(?P<text>[^\]]*)\]\]')


class HorizontalRuleRenderer(PaddedInlineRenderer):
    def __init__(self):
        super().__init__(r'-{4,10}')


class HeadingRenderer(PaddedInlineRenderer):
    def __init__(self):
        super().__init__(r'={1,}[ \t]+(.*?)[ \t]+\={1,}')


class TextDecorationRenderer(InlineRenderer):
    def __init__(self):
        super().__init__(
            r"'''(?P<text>.*?)'''",
            r"''(?P<text>.*?)''",
            r'(~~|--)(.*?)\1',
            r'__(?P<text>.*?)__',
            r'\^\^(?P<text>.*?)\^\^',
            r',,(?P<text>.*?),,',
            r'\{\{\{\+\d+[ \t]+(?P<text>.*?)\}\}\}',
            r'\{\{\{\#(?:[0-9a-f]+|[a-z]+)[ \t]+(?P<text>.*?)\}\}\}')


class MathRenderer(Renderer):
    def __init__(self):
        super().__init__(
            r'<math>(?P<text>.*?)</math>',
            r'\$(?P<text>.*?)\$',
            r'\[math\((?P<text>.*?)\)\]')

    def replacement(self, match: re.Match) -> str:
        text = match.group('text').strip()
        return text if len(text) < 3 else ''


class BoxTextRenderer(PaddedInlineRenderer):
    def __init__(self):
        super().__init__(r'\{\{\|(?P<text>[\s\S]*?)\|\}\}')


class TableRenderer(PaddedInlineRenderer):
    def __init__(self):
        super().__init__(r'\|.*?\|(?:<[^>]*>)*(?P<text>[^\|]*)')


class ListRenderer(PaddedInlineRenderer):
    def __init__(self):
        super().__init__(
            r'^[ \t]+\*[ \t]*(?P<text>.*)$',
            r'^[ \t]+[1AaIi]\.[ \t]*(?:\#\d+)?[ \t]*(?P<text>.*)$')


class QuoteRenderer(PaddedInlineRenderer):
    def __init__(self):
        super().__init__(r'\>+[ \t]*(?P<text>.*)')


class ExtensionRenderer(PaddedInlineRenderer):
    def __init__(self):
        super().__init__(
            r'\{\{\{\#\!syntax[ \t\n]+[\s\S]*?\}\}\}',
            (r'\{\{\{\#\!wiki[ \t\n]+style[ \t]*\=[ \t]*\".*?\"[ \t]*'
             r'(?P<text>[\s\S]*?)\}\}\}'),
            r'\{\{\{\#\!folding[ \t\n]+(?P<text>[\s\S]*?)\}\}\}')


class PlainTextRenderer(PaddedInlineRenderer):
    def __init__(self):
        super().__init__(r'\{\{\{(?P<text>[\s\S]*?)\}\}\}')


class CommentRenderer(PaddedInlineRenderer):
    def __init__(self):
        super().__init__(r'\#\#.*')


class RedirectionRenderer(PaddedInlineRenderer):
    def __init__(self):
        super().__init__(r'\#(?:redirect|넘겨주기)[ \t\n]+.*')


class FootnoteRenderer(Renderer):
    def __init__(self):
        super().__init__(
            r'\[\*[^ \t]+\]',
            r'\[\*[^ \t]*[ \t]+(?P<text>[^\[\]]*)\]')

    def replacement(self, match: re.Match) -> str:
        footnote = match.groupdict().get('text', '').strip()
        return f'({footnote})' if footnote else ''


class NamuWikiRenderer:
    renderers = [
        MacroRenderer(),
        HTMLRenderer(),
        ImageRenderer(),
        HyperLinkRenderer(),
        HorizontalRuleRenderer(),
        HeadingRenderer(),
        TextDecorationRenderer(),
        MathRenderer(),
        BoxTextRenderer(),
        TableRenderer(),
        ListRenderer(),
        QuoteRenderer(),
        ExtensionRenderer(),
        PlainTextRenderer(),
        CommentRenderer(),
        RedirectionRenderer(),
        FootnoteRenderer()]

    @staticmethod
    def render(source: str) -> str:
        for renderer in NamuWikiRenderer.renderers:
            source = renderer.render(source)

        # Remove duplicated spaces.
        while '  ' in source:
            source = source.replace('  ', '')
        source = '\n'.join([line.strip() for line in source.splitlines()])

        return source



import langumo_ko.utils as utils
from langumo_ko.moducorpus.base import ModuBaseParser


class ModuWrittenParser(ModuBaseParser):
    def parse(self, text: str) -> str:
        return text


class ModuWebParser(ModuBaseParser):
    def parse(self, text: str) -> str:
        return text


class ModuNewsParser(ModuBaseParser):
    def parse(self, text: str) -> str:
        text = '\n'.join([line.strip() for line in text.splitlines()
                          if line.strip()])

        # Skip the contents which contain too many non-Korean characters.
        if utils.korean_character_ratio(text) < 0.5:
            return ''

        # Normalize the contents by removing abnormal sentences.
        text = '\n'.join([
            line for line in text.splitlines()
            if utils.is_normal_character(line[0]) and line[-1] == '.'])

        return text



import ijson
from typing import Iterable
from langumo.building import Parser
from langumo.utils import AuxiliaryFile
import langumo_ko.utils as utils
from langumo_ko.namuwiki.rendering import NamuWikiRenderer


class NamuWikiParser(Parser):
    def extract(self, raw: AuxiliaryFile) -> Iterable[str]:
        with raw.open('r') as fp:
            for prefix, event, value in ijson.parse(fp):
                if not prefix.endswith('.text'):
                    continue

                # Skip the redirection pages.
                if value.lower().strip().startswith('#redirect'):
                    continue

                yield value

    def parse(self, text: str) -> str:
        text = NamuWikiRenderer.render(text)
        return '\n'.join([line for line in text.splitlines()
                          if utils.korean_character_ratio(line) > 0.1
                          and '.' in line])



from setuptools import setup, find_packages

setup(
    name='langumo-ko',
    version='0.1.2',

    author='Jungwoo Park',
    author_email='affjljoo3581@gmail.com',

    description='한국어 말뭉치용 langumo parser 모음',
    long_description=open('README.md', 'r', encoding='utf-8').read(),
    long_description_content_type='text/markdown',

    keywords=['langumo', 'corpus', 'dataset', 'nlp', 'language-model',
              'deep-learning', 'machine-learning', 'korean'],
    url='https://github.com/affjljoo3581/langumo-ko',
    license='Apache-2.0',

    package_dir={'': 'src'},
    packages=find_packages('src'),
    python_requires='>=3.6.0',
    install_requires=[
        'langumo',
        'ijson>=3.1.1'
    ],

    classifiers=[
        'Environment :: Console',
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ]
)
